--[[
WarlockReminder Continued v1.20
06 Dec 2010
Author: CKlausi/Biny of Anetheron EU
Original Author of WarlockReminder: Vux of Dragonblight-US
Description: Reminds you of important Warlock-Buffs + Combatreminders
File: WarlockReminder.lua
]]--

WarlockReminder = LibStub("AceAddon-3.0"):NewAddon("WarlockReminder", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("WarlockReminder")

-- Default Values
local defaults = {
    profile = {
		--Reminder
		showArmor = true,
		showSL = true,
		showFlask = true,
		showPet = true,
		showFood = true,
		-- DS
		playDrainSoul = true,
		showDrainSoulMessage = true,
		drainSoulPercent = 25 / 100,
		drainSoulHealth = 100000,
		showDrainSoulFrame = true,
		drainSoulDuration = 5,
		-- Output
		outputFrame = "wlrframe",
		-- Anchor
		wlrAnchorTop = UIParent:GetHeight()/2, 
		wlrAnchorLeft = UIParent:GetWidth()/2,
		wlrAnchorAlpha = 1,
		wlrAnchorSize = 1,
		-- Decimation
		playDecimationSound = true,
		showDecimationMessage = true,
		showDecimationFrame = true,
		-- MoltenCore
		playMoltenCoreSound = false,
		showMoltenCoreMessage = false,
		showMoltenCoreFrame = false,
		-- Shadow Trance
		playShadowTranceSound = false,
		showShadowTranceMessage = false,
		showShadowTranceFrame = false,
		-- Empowered Imp
		playEmpoweredImpSound = false,
		showEmpoweredImpMessage = false,
		showEmpoweredImpFrame = false,
		-- Dark Intent
		showDarkIntentFrame = true,
		diFrameTop = (UIParent:GetHeight()/2) +40,
		diFrameLeft = (UIParent:GetWidth()/2),
		diFrameAlpha = 1,
		diBgAlpha = 0.5,
		diFrameSize = 1,
		diFrameLock = false,
		diFrameShowRealm = false,
		diExtraWidth = 0,
		diFontSize = 11,
    },
}
-- END Default Values

-- Spell Ids
FelArmor = 28176
DemonArmor = 687
SoulLink = 19028
DrainSoul = 1120
UnendingBreath = 5697

--Wotlk
FlaskOfFrostwyrm = 53755
SpellpowerElixir = 33721
--Cata
ElixirOfTheMaster = 79635
ElixirOfTheMightySpeed = 79632
ElixirOfImpossibleAccuracy = 79481
ElixirOfTheCobra = 79477
FlaskOfTheDraconicMind = 79470

WellFed = 57399 -- there are so many different buffs - using textures

-- Pets
Imp = 688
Voidwalker = 697
Succubus = 712
Felhunter = 691
Felguard = 30146

-- Procs
Decimation1 = 63165
Decimation2 = 63167
MoltenCore1 = 47383
MoltenCore2 = 71162
MoltenCore3 = 71165
ShadowTrance = 17941
EmpoweredImp = 54278

-- Dark Intent
DarkIntentSpell = 80398
DarkIntent = 85767
-- END Spell Ids

DarkIntentTarget = ""
timeLeft = 0

-- Addon start
function WarlockReminder:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("WarlockReminderDB", defaults, "Default")
	
	self:CreateFrames()
	self:DiFrameSetup()
	
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	self.db.RegisterCallback(self, "OnNewProfile", "NewConfig")
	
	self:SetUpOptions()
	
	DarkIntentTimer = self:ScheduleRepeatingTimer("DarkIntentTracker", 2)
end


function WarlockReminder:OnEnable()
	-- Only show warnings when player is a warlock
	local localizedClass, englishClass = UnitClass("player")
	DrainSoulPlayed = false
	DecimationPlayed = false
	ReminderPlayed = false
	if ( englishClass == "WARLOCK") then
		self:RegisterEvent("PLAYER_TARGET_CHANGED")
		self:RegisterEvent("UNIT_COMBAT")
		self:RegisterEvent("UNIT_AURA")
		self:RegisterEvent("UNIT_PET")
		self:RegisterEvent("PLAYER_LEAVE_COMBAT")
		self:RegisterEvent("COMPANION_UPDATE")
		self:RegisterEvent("UNIT_SPELLCAST_SENT")
	end
	
	-- Anchor Position
	if (not self.db.profile.wlrAnchorLeft and not self.db.profile.wlrAnchorTop) then
		wlranchor:Show()
		-- When no changes are made, set left and top of the anchor
		self:SetWlrAnchorLeft(self:GetLeft())
		self:SetWlrAnchorTop(self:GetTop())
	else
		wlranchor:ClearAllPoints()
		wlranchor:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT", self.db.profile.wlrAnchorLeft, self.db.profile.wlrAnchorTop)
		wlrframe:SetAlpha(self.db.profile.wlrAnchorAlpha)
		wlrframe:SetWidth(48*self.db.profile.wlrAnchorSize)
		wlrframe:SetHeight(48*self.db.profile.wlrAnchorSize)
	end
	
	-- DiFrame Position
	if (not self.db.profile.diFrameLeft and not self.db.profile.diFrameTop) then
		diFrame:Show()
		-- When no changes are made, set left and top of the frame
		self:SetDiFrameLeft(self:GetLeft())
		self:SetDiFrameTop(self:GetTop())
	else
		self:DiFrameSetup()
	end
end
-- END Addon start

-- Check if the unit has a buff, showcount = true returns how often the buff is up
local function UnitHasBuff(unit, buff, showcount)
    for i=1, 40 do
		local name, rank, iconTexture, count = UnitBuff(unit, i)
        if ( name == GetSpellInfo(buff) ) then
			if(showcount == true) then return count
			else return true end
        end
    end
	return false
end

-- Check if the unit has a buff using textures
local function UnitHasBuffByTexture(unit, buff)
	local  _, _, icon = GetSpellInfo(buff)
    for i=1, 40 do
		local _, _, iconTexture = UnitBuff(unit, i)
        if ( iconTexture == icon  ) then
			return true
        end
    end
	return false
end

-- Output Blizzard Message or Chat
function WarlockReminder:Output(input, red, green, blue, duration)
	-- Simulate default values, is there a better way to do this in lua? )
	if red == nil then red = 0.0 end
	if green == nil then green = 1.0 end
	if blue == nil then blue = 0.0 end
	if duration == nil then duration = 5.0 end
	
	if self.db.profile.outputFrame == "chat" then
		self:Print(input)
	else
		UIErrorsFrame:AddMessage(input, red, green, blue, duration)
	end	
end
-- END Output

-- EVENTS
function WarlockReminder:UNIT_SPELLCAST_SENT(event, unit, spell, _, target)
	if(unit == "player" and spell == GetSpellInfo(DarkIntent)) then
		DarkIntentTarget = target
	end
end

function WarlockReminder:PLAYER_TARGET_CHANGED()
	DrainSoulPlayed = false
	ReminderPlayed = false
	DrainSoulShown = false
	self:DrainSoulReminder()
	self:Reminder()
end

function WarlockReminder:PLAYER_LEAVE_COMBAT()
	self:ReminderPlayed()
	self:Reminder()
end

function WarlockReminder:UNIT_COMBAT()
--	self:ReminderPlayed()
	self:DrainSoulReminder()
end

function WarlockReminder:UNIT_AURA(event, unit)
	if unit == "player" then
		self:ReminderPlayed()
		self:MoltenCoreReminder()
		self:DecimationReminder()
		self:ShadowTranceReminder()
		self:EmpoweredImpReminder()
		self:Reminder()
	end
end

function WarlockReminder:UNIT_PET(event, unit)
	if unit == "player" then
		self:Reminder()
	end
end

function WarlockReminder:COMPANION_UPDATE(event, companion)
	if companion == "MOUNT" then
		self:Reminder()
	end
end
-- END EVENTS

function WarlockReminder:ReminderPlayed()

	if ( not UnitHasBuff("player", ShadowTrance) 
	) then
		ShadowTrancePlayed = false
		if ( not DrainSoulPlayed and
			not DrainSoulShown 
		) then
			self:WlrFrame(false)
		end
	end
	if ( not UnitHasBuff("player", EmpoweredImp) 
	) then
		EmpoweredImpPlayed = false
		if ( not DrainSoulPlayed and
			not DrainSoulShown 
		) then
			self:WlrFrame(false)
		end
	end
	if ( not UnitHasBuff("player", Decimation1) and
		not UnitHasBuff("player", Decimation2) 
	) then
		DecimationPlayed = false
	end
	if ( not UnitHasBuff("player", MoltenCore1) and
		not UnitHasBuff("player", MoltenCore2) and
		not UnitHasBuff("player", MoltenCore3)
	) then
		MoltenCorePlayed = false
	end

end

-- Warlock reminder
function WarlockReminder:Reminder()
	
	missing = false
	
	-- Player not in vehicle and not dead
	if( not UnitInVehicle("player") and
		not UnitIsDead("player")
	) then
		if ( not DrainSoulShown and -- don't show when drain soul is up
			not DrainSoulPlayed and
			not DecimationPlayed and -- don't show when decimation is up
			not MoltenCorePlayed and -- don't show when molten core is up
			not ShadowTrancePlayed and -- don't show when shadow trance is up
			not EmpoweredImpPlayed -- don't show when shadow trance is up
		) then
			-- Bufffood
			if ( self.db.profile.showFood and
				UnitLevel("target") < 0 and
				UnitHealth("target") * 100 / UnitHealthMax("target") > 98 and
				not UnitHasBuffByTexture("player", WellFed) ) then
					missing = true
					if self.db.profile.outputFrame == "wlrframe" then
						self:WlrFrame(true, WellFed)
					else
						self:Output("* ".. L["Missing Food"] .." *")
					end
			else
				if not missing then self:WlrFrame(false) end
			end
			
			-- Flask & Elixir
			if ( self.db.profile.showFlask and
				UnitLevel("target") < 0 and
				UnitHealth("target") * 100 / UnitHealthMax("target") > 98 and
				-- Wotlk
				not UnitHasBuff("player", FlaskOfFrostwyrm) and
				not UnitHasBuff("player", SpellpowerElixir) and
				-- Cata
				not UnitHasBuff("player", ElixirOfTheMaster) and
				not UnitHasBuff("player", ElixirOfTheMightySpeed) and
				not UnitHasBuff("player", ElixirOfImpossibleAccuracy) and
				not UnitHasBuff("player", ElixirOfTheCobra) and
				not UnitHasBuff("player", FlaskOfTheDraconicMind) ) then
					missing = true
					if self.db.profile.outputFrame == "wlrframe" then
						self:WlrFrame(true, FlaskOfTheDraconicMind)
					else
						self:Output("* ".. L["Missing Flask"] .." *")
					end
			else
				if not missing then self:WlrFrame(false) end
			end
			
			-- Soul Link
			local slusable, slnomana = IsUsableSpell(SoulLink)
			if( self.db.profile.showSL and
				UnitExists("pet") and
				slusable == 1 and -- Spell already learned?
				not IsMounted("player") ) then
					if( not UnitHasBuff("player", SoulLink) ) then
						missing = true
						if self.db.profile.outputFrame == "wlrframe" then
							self:WlrFrame(true, SoulLink)
						else
							self:Output("* ".. L["Missing SL"] .." *")
						end
					end
			else
				if not missing then self:WlrFrame(false) end
			end
			
			-- Pet
			if ( self.db.profile.showPet and
				not UnitExists("pet") and
				not IsMounted("player") ) then
					missing = true
					if self.db.profile.outputFrame == "wlrframe" then
						local pet = ""
						local talents = GetPrimaryTalentTree(false, false, GetActiveTalentGroup(false, false))
						if talents == 1 then pet = Succubus -- Affli
						elseif (talents == 2 and IsUsableSpell(Felguard) == 1) then pet = Felguard -- Demo
						elseif (talents == 2 and IsUsableSpell(Felguard) == nil) then pet = Voidwalker -- Demo without Felguard
						else pet = Imp end -- Destro + unknown
						self:WlrFrame(true, pet)
					else
						self:Output("* ".. L["Missing Pet"] .." *")
					end
			else
				if not missing then	self:WlrFrame(false) end
			end

			-- Armor
			local felarmorusable, felarmornomana = IsUsableSpell(FelArmor)
			local demonarmorusable, demonarmornomana = IsUsableSpell(DemonArmor)
			if ( self.db.profile.showArmor and
				-- Spell already learned?
				( felarmorusable == 1 or
				demonarmorusable == 1 ) and
				not UnitHasBuff("player", FelArmor) and -- Fel Armor
				not UnitHasBuff("player", DemonArmor)  -- Demon Armor
			) then
				missing = true
				if self.db.profile.outputFrame == "wlrframe" then
					local usearmor = ""
					if felarmorusable == 1 then usearmor = FelArmor
					else usearmor = DemonArmor end
					self:WlrFrame(true, usearmor)
				else 
					self:Output("* ".. L["Missing Armor"] .." *")
				end
			else
				if not missing then self:WlrFrame(false) end
			end
		end
	else
		self:WlrFrame(false)
	end
end
-- END Warlock reminder

-- DrainSoul reminder
function WarlockReminder:DrainSoulReminder()
	local dsusable, dsnomana = IsUsableSpell(DrainSoul)
	if ( not UnitInVehicle("player") and
		not ShadowTrancePlayed and -- don't show when Shadow Trance is up
		dsusable == 1 and -- Spell already learned?
		UnitCanAttack("player", "target") and
		not UnitIsDead("target") and
		UnitHealth("target") > 0 and
		UnitHealth("target") * 100 / UnitHealthMax("target") <= self.db.profile.drainSoulPercent * 100 and
		UnitHealthMax("target") >= tonumber(self.db.profile.drainSoulHealth)
		) then
			-- play ds sound
			if ( self.db.profile.playDrainSoul and
				not DrainSoulPlayed
				) then
				PlaySoundFile("Interface\\AddOns\\WarlockReminder\\Sounds\\quaddamage.ogg")
			end
			-- show ds message
			if ( self.db.profile.showDrainSoulMessage and 
				not DrainSoulPlayed
				) then
				self:Output("* ".. L["Drain Soul!"] .." *", 1.0, 0.0, 0.0)
			end
			-- show ds wlrframe
			if self.drainsoultimer then local temptimer = self.drainsoultimer 
			else temptimer = nil end
			if ( self.db.profile.showDrainSoulFrame and
				not self:TimeLeft(temptimer) and
				not DrainSoulShown
			) then
				DrainSoulShown = true
				self:WlrDrainSoulFrame(true)
			end
			DrainSoulPlayed = true
	else 
		DrainSoulPlayed = false
		DrainSoulShown = false
	end
end
-- END DrainSoul reminder

-- Decimation reminder
function WarlockReminder:DecimationReminder()
	if( ( UnitHasBuff("player", Decimation1) or
		UnitHasBuff("player", Decimation2) )
	) then
		-- play decimation sound
		if ( self.db.profile.playDecimationSound and
			not DecimationPlayed
		) then
			PlaySoundFile("Interface\\AddOns\\WarlockReminder\\Sounds\\quaddamage.ogg")
		end
		-- show decimation message
		if ( self.db.profile.showDecimationMessage and 
			not DecimationPlayed
		) then
			self:Output("* ".. L["Decimation!"] .." *", 1.0, 0.0, 0.0)
		end
		-- show decimation frame
		if  self.db.profile.showDecimationFrame then
			self:WlrFrame(true, Decimation2)
		end
		DecimationPlayed = true
	else
		DecimationPlayed = false
	end
end
-- END Decimation reminder

-- moltencore reminder
function WarlockReminder:MoltenCoreReminder()
	if( ( UnitHasBuff("player", MoltenCore1) or
		UnitHasBuff("player", MoltenCore2) or
		UnitHasBuff("player", MoltenCore3) ) and
		not DecimationPlayed  -- don't show when decimation is up
	) then
		-- play moltencore sound
		if ( self.db.profile.playMoltenCoreSound and
			not MoltenCorePlayed
		) then
			PlaySoundFile("Sound\\Spells\\Clearcasting_Impact_Chest.wav")
		end
		-- show moltencore message
		if ( self.db.profile.showMoltenCoreMessage and 
			not MoltenCorePlayed
		) then
			self:Output("* ".. L["Molten Core!"] .." *", 1.0, 0.0, 0.0)
		end
		-- show moltencore frame
		if  self.db.profile.showMoltenCoreFrame then
			-- How often Molten Core is up
			local count = 0
			if UnitHasBuff("player", MoltenCore1) then count = UnitHasBuff("player", MoltenCore1, true)
			elseif UnitHasBuff("player", MoltenCore2) then count = UnitHasBuff("player", MoltenCore2, true)
			else count = UnitHasBuff("player", MoltenCore3, true) end
			
			self:WlrMoltenCoreFrame(true, count)
		end
		MoltenCorePlayed = true
	else
		MoltenCorePlayed = false
	end
end
-- END moltencore reminder

-- ShadowTrance reminder
function WarlockReminder:ShadowTranceReminder()
	if( UnitHasBuff("player", ShadowTrance)
	) then
		-- play shadowtrance sound
		if ( self.db.profile.playShadowTranceSound and
			not ShadowTrancePlayed
		) then
			PlaySoundFile("Sound\\Spells\\Clearcasting_Impact_Chest.wav")
		end
		-- show shadowtrance message
		if ( self.db.profile.showShadowTranceMessage and 
			not ShadowTrancePlayed
		) then
			self:Output("* ".. L["Shadow Trance!"] .." *", 1.0, 0.0, 0.0)
		end
		-- show shadowtrance frame
		if  self.db.profile.showShadowTranceFrame then
			self:WlrFrame(true, ShadowTrance)
		end
		ShadowTrancePlayed = true
	else
		ShadowTrancePlayed = false
	end
end
-- END ShadowTrance reminder

-- EmpoweredImp reminder
function WarlockReminder:EmpoweredImpReminder()
	if(  UnitHasBuff("player", EmpoweredImp)
	) then
		-- play empowered imp sound
		if ( self.db.profile.playEmpoweredImpSound and
			not EmpoweredImpPlayed
			) then
			PlaySoundFile("Sound\\Spells\\Clearcasting_Impact_Chest.wav")
		end
		-- show empowered imp message
		if ( self.db.profile.showEmpoweredImpMessage and 
			not EmpoweredImpPlayed
			) then
			self:Output("* ".. L["Empowered Imp!"] .." *", 1.0, 0.0, 0.0)
		end
		-- show empowered imp frame
		if  self.db.profile.showEmpoweredImpFrame then
			self:WlrFrame(true, EmpoweredImp)
		end
		EmpoweredImpPlayed = true
	else
		EmpoweredImPlayed = false
	end
end
-- END EmpoweredImp reminder

-- Dark Intent Tracker
function WarlockReminder:DarkIntentTracker()
	local output = 0
	if(	(UnitInRaid(DarkIntentTarget) or UnitInParty(DarkIntentTarget)) and
		UnitHasBuff(DarkIntentTarget, DarkIntent)
	)then
		local _, englishClass = UnitClass(DarkIntentTarget)
		local fontColor
		local diTarget, diName
		
		if (not (DarkIntentTarget == "")) then
			local buffName, _, _, _, _, _, expirationTime, _, _ = UnitBuff(DarkIntentTarget, GetSpellInfo(DarkIntent))
			timeLeft = expirationTime
		end
		
		if((-1*(GetTime()-timeLeft)) < 0) then output = ""
		elseif((-1*(GetTime()-timeLeft)) >= 60) then output = format("%.0f",ceil(-1*(GetTime()-timeLeft)/60)).." "..L["minutes"]
		else output = format("%.0f",-1*(GetTime()-timeLeft)).." "..L["seconds"] end
		
		if(self.db.profile.diFrameShowRealm == true) then
			diName = DarkIntentTarget
		else
			diTarget, diRealm = strsplit("-", DarkIntentTarget)
			diName = diTarget
		end
		
		if RAID_CLASS_COLORS[englishClass] then
			fontColor = RAID_CLASS_COLORS[englishClass]
			diFontFrame.text:SetTextColor(fontColor["r"], fontColor["g"], fontColor["b"])
			diFontFrame.textTime:SetText(output)
		else
			diFontFrame.textTime:SetText(output)
		end
		diFontFrame.text:SetText(diName)
		self:DiFrameSetFontGab()
	else
		timeLeft = 0
		diFontFrame.text:SetTextColor(1, 1, 1)
		diFontFrame.text:SetText(L["No Dark Intent"])
		diFontFrame.textTime:SetText("")
	end
end
-- END Dark Intent Tracker
